
# q-expansion of newform 3200.2.a.g, downloaded from the LMFDB on 29 May 2024.

# We generate the q-expansion using the Hecke eigenvalues a_p at the primes.
# Each a_p is given as a linear combination
# of the following basis for the coefficient ring.

def make_data():

    from sage.all import prod, floor, prime_powers, gcd, QQ, primes_first_n, next_prime, RR

    def discrete_log(elts, gens, mod):
        # algorithm 2.2, page 16 of https://arxiv.org/abs/0903.2785
        def table_gens(gens, mod):
            T = [1]
            n = len(gens)
            r = [None]*n
            s = [None]*n
            for i in range(n):
                beta = gens[i]
                r[i] = 1
                N = len(T)
                while beta not in T:
                    for Tj in T[:N]:
                        T.append((beta*Tj) % mod)
                    beta = (beta*gens[i]) % mod
                    r[i] += 1
                s[i] = T.index(beta)
            return T, r, s
        T, r, s = table_gens(gens, mod)
        n = len(gens)
        N = [ prod(r[:j]) for j in range(n) ]
        Z = lambda s: [ (floor(s/N[j]) % r[j]) for j in range(n)]
        return [Z(T.index(elt % mod)) for elt in elts]
    def extend_multiplicatively(an):
        for pp in prime_powers(len(an)-1):
            for k in range(1, (len(an) - 1)//pp + 1):
                if gcd(k, pp) == 1:
                    an[pp*k] = an[pp]*an[k]
    from sage.all import PolynomialRing, NumberField
    R = PolynomialRing(QQ, "x")
    f = R(poly_data)
    K = NumberField(f, "a")
    betas = [K.gens()[0]**i for i in range(len(poly_data))]
    convert_elt_to_field = lambda elt: sum(c*beta for c, beta in zip(elt, betas))
    # convert aps to K elements
    primes = primes_first_n(len(aps_data))
    good_primes = [p for p in primes if not p.divides(level)]
    aps = map(convert_elt_to_field, aps_data)
    if not hecke_ring_character_values:
        # trivial character
        char_values = dict(zip(good_primes, [1]*len(good_primes)))
    else:
        gens = [elt[0] for elt in hecke_ring_character_values]
        gens_values = [convert_elt_to_field(elt[1]) for elt in hecke_ring_character_values]
        char_values = dict([(
            p,prod(g**k for g, k in zip(gens_values, elt)))
            for p, elt in zip(good_primes, discrete_log(good_primes, gens, level))
            ])
    an_list_bound = next_prime(primes[-1])
    an = [0]*an_list_bound
    an[1] = 1
    
    from sage.all import PowerSeriesRing
    PS = PowerSeriesRing(K, "q")
    for p, ap in zip(primes, aps):
        if p.divides(level):
            euler_factor = [1, -ap]
        else:
            euler_factor = [1, -ap, p**(weight - 1) * char_values[p]]
        k = RR(an_list_bound).log(p).floor() + 1
        foo = (1/PS(euler_factor)).padded_list(k)
        for i in range(1, k):
            an[p**i] = foo[i]
    extend_multiplicatively(an)
    return PS(an)
level = 3200
weight = 2
poly_data = [0, 1]

# The basis for the coefficient ring is just the power basis
# in the root of the defining polynomial above.
hecke_ring_character_values = None
aps_data = [[0], [-2], [0], [0], [2], [2], [-6], [6], [0], [-10], [8], [2], [-6], [2], [-12], [10], [-6], [6], [14], [4], [10], [-8], [-10], [14], [-6], [6], [8], [2], [-2], [-2], [12], [6], [6], [-14], [6], [12], [10], [14], [16], [10], [-2], [14], [24], [-6], [18], [28], [-10], [20], [6], [14], [10], [8], [-2], [18], [-18], [-24], [-2], [-32], [2], [30], [-14], [-6], [22], [28], [-26], [10], [2], [-2], [-30], [22], [-2], [12], [-28], [18], [26], [4], [-34], [-38], [-2], [-22], [30], [-10], [0], [26], [20], [2], [14], [6], [22], [28], [6], [-16], [-16], [-22], [-2], [16], [14], [10], [-6], [-34], [-2], [-22], [-34], [-38], [-22], [-18], [18], [14], [-28], [14], [28], [-14], [-6], [42], [-4], [-2], [6], [-16], [10], [30], [38], [-22], [-38], [18], [-10], [-42], [6], [-8], [-8], [-30], [6], [-8], [-32], [42], [10], [-50], [10], [-18], [-14], [26], [-46], [-18], [-16], [-6], [-10], [-4], [58], [38], [26], [4], [18], [2], [-18], [8], [-22], [48], [20], [-18], [-38], [22], [-42], [6], [-32], [42], [10], [-48], [-24], [42], [-18], [18], [18], [-26], [-28], [-54], [40], [30], [50], [22], [40], [-10], [28], [-34], [34], [58], [12], [30], [26], [46], [-6], [-8], [-66], [2], [30], [-18], [38], [42], [-62], [2], [30], [-8], [-18], [32], [-22], [50], [10], [18], [16], [-34], [30], [-14], [10], [54], [16], [42], [-36], [-22], [68], [-2], [-8], [-54], [22], [-44], [-50], [-52], [54], [-34], [10], [0], [-16], [10], [-6], [6], [-56], [-18], [-46], [52], [-50], [-62], [18], [36], [-26], [10], [72], [46], [-2], [-28], [-52], [38], [50], [12], [-54], [-78], [64], [14], [66], [30], [-18], [18], [18], [-42], [-44], [-42], [-26], [34], [-22], [78], [-26], [-2], [-6], [-78], [14], [-58], [-54], [32], [46], [16], [42], [-2], [-34], [-34], [-28], [-60], [-24], [14], [58], [64], [-38], [42], [20], [50], [-82], [38], [-54], [34], [50], [-10], [-8], [-6], [34], [-50], [-26], [-6], [-8], [-26], [-70], [10], [-62], [30], [-76], [2], [28], [-2], [30], [22], [32], [-34], [-34], [-24], [-34], [66], [-10], [38], [30], [-28], [22], [-2], [-26], [50], [-20], [-14], [22], [66], [80], [-42], [58], [-46], [30], [-66], [-50], [52], [-54], [22], [-18], [-12], [-54], [-26], [38], [-46], [0], [50], [-50], [70], [-58], [-52], [46], [-54], [48], [42], [74], [-80], [90], [-50], [-12], [-70], [22], [42], [-14], [-40], [26], [-90], [42], [-36], [14], [-4], [-38], [30], [-8], [14], [-82], [-42], [-42], [6], [-56], [78], [22], [-24], [-32], [-46], [26], [-68], [-46], [50], [58], [62], [-52], [-58], [-64], [-54], [-22], [-2], [14], [-54], [20], [-42], [-90], [-12], [-62], [62], [38], [62], [46], [34], [-6], [46], [-6], [30], [-80], [-64], [-38], [-16], [6], [-22], [-92], [42], [6], [42], [-82], [30], [-22], [36]]
